#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: caktux
# @Date:   2014-12-21 08:28:41
# @Last Modified by:   caktux
# @Last Modified time: 2014-12-21 11:42:56

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions
