
# This file was generated by 'versioneer.py' (0.12) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

version_version = '0.7.0-2-g0c0928f'
version_full = '0c0928f8ff7674bfec725ba4c873fa18a35d8e8e'
def get_versions(default={}, verbose=False):
    return {'version': version_version, 'full': version_full}

